from setuptools import setup


setup(name='qservice',
      version='0.2',
      description='Standardize methods and create step-by-step flows',
      url='https://github.com/rvettori/qservice',
      author='Rafael Vettori',
      author_email='rafael.vettori@gmail.com',
      license='MIT',
      packages=['qservice'],
      zip_safe=False)
